package com.gx.obe.step.price.calculator;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;
import java.util.function.Function;

import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.step.price.beans.SupplierPriceDetailEvalResult;
import com.gx.obe.step.price.utils.FactorUtils;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.entity.EvaluationFactorResult;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;

/**
 * @Description: 供应商价格评审结果计算器
 * @author chenxw
 */
public class SupplierPriceEvalResultCalculator {
	
	private final String tenderId;
	private final String stepId;
	private final String userId;
	
	private final EvaluationFactor evaluationFactor;
	
	/**
	 * @Description: 计算最终得分函数
	 */
	private final Function<BigDecimal, BigDecimal> calcuateFinalScoreFun;
	
	/**
	 * @Description: 供应商总分计算器
	 */
	private SupplierTotalScoreCalculator supplierTotalScoreCalculator;
	
	/**
	 * @author chenxw
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @param evaluationFactor
	 */
	public SupplierPriceEvalResultCalculator(String tenderId, String stepId, String userId, EvaluationFactor evaluationFactor) {
		this.tenderId = tenderId;
		this.stepId = stepId;
		this.userId = userId;
		this.evaluationFactor = evaluationFactor;
		
		Objects.requireNonNull(tenderId);
		Objects.requireNonNull(stepId);
		Objects.requireNonNull(userId);
		Objects.requireNonNull(evaluationFactor);
		
		this.calcuateFinalScoreFun = FactorUtils.getRealWeight(evaluationFactor.getFactorFinalWeight())::multiply;
		this.supplierTotalScoreCalculator = SupplierTotalScoreCalculatorFactory.create(PropertiesUtils.getProperty(ConfigConstants.SUPPLIER_TOTAL_SCORE_CALCULATE_TYPE));
	}
	
	/**
	 * @author chenxw
	 * @Description: 计算价格指标评审结果（明细汇总结果）
	 * @param supplierId
	 * @param supplierPriceDetailEvalResultList
	 * @return
	 */
	public EvaluationFactorResult calculate(String supplierId, List<SupplierPriceDetailEvalResult> supplierPriceDetailEvalResultList) {
		Objects.requireNonNull(supplierId);
		CollectionUtils.notNull(supplierPriceDetailEvalResultList);
		CollectionUtils.notContainNull(supplierPriceDetailEvalResultList);
		
		EvaluationFactorResult efr = new EvaluationFactorResult();
		efr.setId(UuidUtils.getUUID());
		efr.setTenderId(tenderId);
		efr.setStepId(stepId);
		efr.setUserId(userId);
		
		efr.setSupplierId(supplierId);
		efr.setFactorId(evaluationFactor.getId());
		efr.setFactorName(evaluationFactor.getFactorName());
		BigDecimal score = supplierTotalScoreCalculator.calculate(supplierPriceDetailEvalResultList);
		efr.setEvaluationScore(score);
		efr.setEvaluationFinalScore(calcuateFinalScoreFun.apply(score));
		return efr;
	}
	
}
