package com.gx.obe.step.price.creater;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.components.core.enumeration.EvaluationStepEnum;
import com.gx.obe.components.core.enumeration.PriceReviewTypeEnum;
import com.gx.obe.datastruct.tree.Forest;
import com.gx.obe.message.Messages;
import com.gx.obe.step.price.enums.PriceDetailTableColEnum;
import com.gx.obe.step.price.utils.FactorUtils;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;

public class DefaultPriceEvalStepCreater {

	private String tenderId;
	private Map<Boolean, List<EvaluationFactor>> priceFactorListMap;

	public DefaultPriceEvalStepCreater(String tenderId, List<EvaluationFactor> priceFactorList) {
		this.tenderId = tenderId;
		this.priceFactorListMap = new Forest<>(priceFactorList).getLeafList().stream().collect(Collectors.partitioningBy(t -> PriceDetailTableColEnum.contain(t.getBidPriceCode(), PriceDetailTableColEnum.TypeEnum.PRICE)));
	}

	public List<PriceEvalStep> createSingle() {
		List<PriceEvalStep> evalStepList = new ArrayList<>();
		createDetail(evalStepList);
		List<EvaluationFactor> evaluationFactorList = priceFactorListMap.get(false);
		if (!evaluationFactorList.isEmpty()) {
			PriceEvalStep evalStep = create();
			evalStep.getPriceStep().setStepName(Messages.Step_price);
			evalStep.getPriceStep().setScore(evaluationFactorList.stream().map(t -> t.getFactorScore()).reduce(null, BigDecimalUtils::add));
			evalStep.getPriceStep().setWeight(evaluationFactorList.stream().map(this::getPriceStepWeight).reduce(null, BigDecimalUtils::add));
			evalStep.getPriceStep().setEvaluationParam(EvaluationStepEnum.PRICE_REVIEW_TYPE, PriceReviewTypeEnum.PRICE);
			evalStep.getFactorList().addAll(evaluationFactorList);
			evalStepList.add(evalStep);
		}
		return evalStepList;
	}

	public List<PriceEvalStep> createMulty() {
		List<PriceEvalStep> evalStepList = new ArrayList<>();
		createDetail(evalStepList);
		for (EvaluationFactor evaluationFactor : priceFactorListMap.get(false)) {
			PriceEvalStep evalStep = create();
			evalStep.getPriceStep().setStepName(evaluationFactor.getFactorName());
			evalStep.getPriceStep().setScore(evaluationFactor.getFactorScore());
			evalStep.getPriceStep().setWeight(getPriceStepWeight(evaluationFactor));
			evalStep.getPriceStep().setEvaluationParam(EvaluationStepEnum.PRICE_REVIEW_TYPE, PriceReviewTypeEnum.PRICE);
			evalStep.getFactorList().add(evaluationFactor);
			evalStepList.add(evalStep);
		}
		return evalStepList;
	}

	private void createDetail(List<PriceEvalStep> evalStepList) {
		for (EvaluationFactor evaluationFactor : priceFactorListMap.get(true)) {
			PriceEvalStep evalStep = create();
			evalStep.getPriceStep().setStepName(evaluationFactor.getFactorName());
			evalStep.getPriceStep().setScore(evaluationFactor.getFactorScore());
			evalStep.getPriceStep().setWeight(getPriceStepWeight(evaluationFactor));
			evalStep.getPriceStep().setEvaluationParam(EvaluationStepEnum.PRICE_REVIEW_TYPE, PriceReviewTypeEnum.DETAIL);
			evalStep.getFactorList().add(evaluationFactor);
			evalStepList.add(evalStep);
		}
	}
	
	private PriceEvalStep create() {
		EvaluationStep priceStep = new EvaluationStep();
		List<EvaluationFactor> factorList = new ArrayList<>();
		priceStep.setId(UuidUtils.getUUID());
		priceStep.setTenderId(tenderId);
		priceStep.setStepType(EvaluationStepEnum.PRICE);
		priceStep.setFilterType(EvaluationStepEnum.FILTER_NO);
		priceStep.setStepStatus(CommonEnum.NOT_START);
		priceStep.setEvalBidType(EvaluationFactorEnum.EVAL_BID_PRICE);
		priceStep.setRoleType(EvaluationStepEnum.EXPERT_LEADER);
		
		PriceEvalStep priceEvalStep = new PriceEvalStep();
		priceEvalStep.setPriceStep(priceStep);
		priceEvalStep.setFactorList(factorList);
		return priceEvalStep;
	}
	
	private BigDecimal getPriceStepWeight(EvaluationFactor evaluationFactor) {
		return evaluationFactor.getFactorScore().multiply(FactorUtils.getRealWeight(evaluationFactor.getFactorFinalWeight()));
	}
	
}
