package com.gx.obe.step.price.creater;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.components.core.enumeration.EvaluationStepEnum;
import com.gx.obe.components.core.enumeration.PriceReviewTypeEnum;
import com.gx.obe.datastruct.tree.Forest;
import com.gx.obe.step.price.enums.PriceDetailTableColEnum;
import com.gx.obe.step.price.utils.FactorUtils;
import com.gx.obe.util.utils.AssertUtils;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;

public class PriceEvalStepCreater {
	
	private int priceEvalStepCount;
	
	private List<EvaluationStep> priceEvalStepList;
	
	private List<EvaluationStep> priceDetailEvalStepList;
	
	private Map<Boolean, List<EvaluationFactor>> priceFactorListMap;
	
	private String tenderId;
	
	public PriceEvalStepCreater(String tenderId, List<EvaluationStep> priceStepList, List<EvaluationFactor> factorList) {
		
		this.tenderId = tenderId;
		
		this.priceEvalStepList = priceStepList.stream().filter(t -> PriceReviewTypeEnum.PRICE.equals(t.getEvaluationParamByKey(EvaluationStepEnum.PRICE_REVIEW_TYPE))).collect(Collectors.toList());
		
		this.priceDetailEvalStepList = priceStepList.stream().filter(t -> PriceReviewTypeEnum.DETAIL.equals(t.getEvaluationParamByKey(EvaluationStepEnum.PRICE_REVIEW_TYPE))).collect(Collectors.toList());
		
		this.priceFactorListMap = new Forest<>(factorList).getLeafList().stream().collect(Collectors.partitioningBy(t -> PriceDetailTableColEnum.contain(t.getBidPriceCode(), PriceDetailTableColEnum.TypeEnum.PRICE)));
		
		this.priceEvalStepCount = priceEvalStepList.size();
		
		AssertUtils.isTrue(priceEvalStepCount <= priceFactorListMap.get(false).size());
		
		AssertUtils.isTrue(priceDetailEvalStepList.size() == priceFactorListMap.get(true).size());
	}
	
	public PriceEvalStep create(EvaluationStep modelEvaluationStep) {
		Objects.requireNonNull(modelEvaluationStep);
		
		EvaluationStep priceStep = new EvaluationStep();
		List<EvaluationFactor> factorList = new ArrayList<>();
		
		priceStep.setId(UuidUtils.getUUID());
		
		String priceReviewType = modelEvaluationStep.getEvaluationParamByKey(EvaluationStepEnum.PRICE_REVIEW_TYPE);
		if (PriceReviewTypeEnum.PRICE.equals(priceReviewType)) {
			List<EvaluationFactor> evaluationFactorList;
			if (priceEvalStepCount == 1) {
				evaluationFactorList = priceFactorListMap.get(false);
				priceStep.setStepName(modelEvaluationStep.getStepName());
			} else {
				evaluationFactorList = new ArrayList<>();
				evaluationFactorList.add(priceFactorListMap.get(false).get(0));
				priceStep.setStepName(evaluationFactorList.get(0).getFactorName());
			}
			priceStep.setScore(evaluationFactorList.stream().map(t -> t.getFactorScore()).reduce(null, BigDecimalUtils::add));
			priceStep.setWeight(evaluationFactorList.stream().map(this::getPriceStepWeight).reduce(null, BigDecimalUtils::add));
			factorList.addAll(evaluationFactorList);
			
			priceEvalStepList.remove(modelEvaluationStep);
			priceFactorListMap.get(false).removeAll(evaluationFactorList);
			priceEvalStepCount--;
		} else if (PriceReviewTypeEnum.DETAIL.equals(priceReviewType)) {
			EvaluationFactor evaluationFactor = priceFactorListMap.get(true).get(0);
			
			priceStep.setStepName(evaluationFactor.getFactorName());
			priceStep.setScore(evaluationFactor.getFactorScore());
			priceStep.setWeight(getPriceStepWeight(evaluationFactor));
			factorList.add(evaluationFactor);
			
			priceDetailEvalStepList.remove(modelEvaluationStep);
			priceFactorListMap.get(true).remove(evaluationFactor);
		}
		
		priceStep.setTenderId(tenderId);
		priceStep.setStepType(EvaluationStepEnum.PRICE);
		priceStep.setFilterType(EvaluationStepEnum.FILTER_NO);
		priceStep.setStepStatus(CommonEnum.NOT_START);
		priceStep.setEvalBidType(EvaluationFactorEnum.EVAL_BID_PRICE);
		
		priceStep.setParams(modelEvaluationStep.getParams());
		priceStep.setRoleType(modelEvaluationStep.getRoleType());
		
		PriceEvalStep priceEvalStep = new PriceEvalStep();
		priceEvalStep.setPriceStep(priceStep);
		priceEvalStep.setFactorList(factorList);
		
		return priceEvalStep;
	}
	
	private BigDecimal getPriceStepWeight(EvaluationFactor evaluationFactor) {
		return evaluationFactor.getFactorScore().multiply(FactorUtils.getRealWeight(evaluationFactor.getFactorFinalWeight()));
	}
	
}
