package com.gx.obe.step.price.dialog;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.function.Predicate;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.nebula.widgets.grid.listener.GridCheckedListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.ImageResolver;
import org.eclipse.swt.core.utils.GridUtils;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.components.core.enumeration.EvaluationStepEnum;
import com.gx.obe.components.core.enumeration.PriceReviewTypeEnum;
import com.gx.obe.datastruct.tree.Forest;
import com.gx.obe.step.dialog.EvaluationStepSingleDialog;
import com.gx.obe.step.price.utils.FactorUtils;
import com.gx.obe.step.price.utils.IPriceEvalStep;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.PredicateUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.evaluation.EvaluationFactorService;

/**
 * @Description: 价格明细评审步骤界面
 * @author chenxw
 */
public class PriceDetailEvaluationStepDialog extends EvaluationStepSingleDialog implements IPriceEvalStep {
	
	private final String FACTOR_DATA = "evaluationFactor";
	
	private final EvaluationFactorService evaluationFactorService = new EvaluationFactorService();
	
	private final Image treeExpandPic = ImageResolver.RESOLVER_INSTANCE.loadImage(IconsActivator.PLUGIN_ID, "icons/tree_expend.png");
	private final Image treeClosePic = ImageResolver.RESOLVER_INSTANCE.loadImage(IconsActivator.PLUGIN_ID, "icons/tree_close.png");
	
	private final Image treeParentPic = ImageResolver.RESOLVER_INSTANCE.loadImage(IconsActivator.PLUGIN_ID, "icons/tree_parent.png");
	private final Image treeItemPic = ImageResolver.RESOLVER_INSTANCE.loadImage(IconsActivator.PLUGIN_ID, "icons/tree_item.png");
	
	private Grid factorGrid;
	
	/**
	 * @author chenxw
	 * @param parentShell
	 * @param tenderProject
	 * @param evaluationStep
	 */
	public PriceDetailEvaluationStepDialog(Shell parentShell, TenderProjectEntity tenderProject, EvaluationStep evaluationStep) {
		super(parentShell, tenderProject, evaluationStep);
	}
	
	@Override
	protected String getTipsText() {
		return "自动计算价格明细得分";
	}
	
	@Override
	protected String getStepNameDef() {
		return "价格明细评审";
	}
	
	@Override
	protected String getStepType() {
		return EvaluationStepEnum.PRICE;
	}
	
	@Override
	protected void createFromArea(Composite fromComposite, Composite gridComposite) {
		super.createFromArea(fromComposite, gridComposite);
		
		Composite composite = new Composite(gridComposite, SWT.NONE);
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginHeight = 0;
		gl_composite.marginWidth = 0;
		composite.setLayout(gl_composite);
		
		factorGrid = new Grid(composite, SWT.NONE);
		GridData gd_composite_1 = new GridData(SWT.CENTER, SWT.FILL, false, true, 1, 1);
		gd_composite_1.widthHint = 200;
		factorGrid.setLayoutData(gd_composite_1);
		factorGrid.addListener(SWT.Selection, new GridCheckedListener(false));
		
		GridColumn gridColumn = new GridColumn(factorGrid, SWT.CHECK);
		gridColumn.setWidth(200);
		gridColumn.setWordWrap(true);
		gridColumn.setTree(true);
		gridColumn.setShowTreeTogglePic(true);
		gridColumn.setTreeTogglePic(treeExpandPic, treeClosePic);
		
		initFactorGrid();
	}
	
	/**
	 * @Description: 初始化指标表格界面
	 * @author chenxw
	 */
	private void initFactorGrid() {
		List<EvaluationFactor> evaluationFactorList = evaluationFactorService.getAllEvaluationFactorList(tenderProject.getId(), EvaluationFactorEnum.AUDIT_PRICE);
		if (CollectionUtils.isNull(evaluationFactorList)) {
			return;
		}
		Forest<EvaluationFactor> factorForest = new Forest<>(evaluationFactorList);
		
		filterForestByStepFactor(tenderProject, evaluationStep, factorForest);
		IPriceEvalStep.filterForestByReviewType(factorForest, PriceReviewTypeEnum.DETAIL);
		
		for (EvaluationFactor evaluationFactor : evaluationFactorList) {
			initFactorGridItem(new GridItem(factorGrid, SWT.NONE), evaluationFactor);
		}
	}
	
	/**
	 * @Description: 初始化指标表格行
	 * @author chenxw
	 * @param parentItem
	 * @param evaluationFactor 
	 */
	private void initFactorGridItem(GridItem parentItem, EvaluationFactor evaluationFactor) {
		parentItem.setText(evaluationFactor.getFactorName());
		List<EvaluationFactor> childFactorList = evaluationFactor.getChildFactorList();
		if (CollectionUtils.isNull(childFactorList)) {
			parentItem.setImage(treeItemPic);
			parentItem.setData(FACTOR_DATA, evaluationFactor);
			return;
		}
		parentItem.setExpanded(true);
		parentItem.setCheckVisible(0, false);
		parentItem.setImage(treeParentPic);
		for (EvaluationFactor childFactor : childFactorList) {
			initFactorGridItem(new GridItem(parentItem, SWT.NONE), childFactor);
		}
	}
	
	@Override
	protected void setFromData(EvaluationStep evaluationStep) {
		super.setFromData(evaluationStep);
		
		List<Object> factorIdList = evaluationStepFactorService.getFactorIdLsitByStepId(evaluationStep.getId());
		Predicate<GridItem> gridItemPre = PredicateUtils.map(this::getFactorIdByItem, factorIdList::contains);
		Arrays.stream(factorGrid.getItems()).filter(gridItemPre).forEach(t -> t.setChecked(true));
	}
	
	/**
	 * @Description: 根据表格行获取指标id
	 * @author chenxw
	 * @param item
	 * @return 
	 */
	private String getFactorIdByItem(GridItem item) {
		return Optional.ofNullable(item).map(t -> (EvaluationFactor) t.getData(FACTOR_DATA)).map(EvaluationFactor::getId).orElse(null);
	}
	
	@Override
	protected String valFromData() {
		List<EvaluationFactor> checkedFactorList = GridUtils.getCheckedValues(factorGrid, 0, FACTOR_DATA, true);
		if (CollectionUtils.isNull(checkedFactorList)) return "请选择评审指标！";
		if (checkedFactorList.size() > 1) return "价格明细评审步骤只能选择一个价格明细指标！";
		return null;
	}
	
	@Override
	protected void getFromData(EvaluationStep evaluationStep) {
		super.getFromData(evaluationStep);
		
		List<EvaluationFactor> checkedFactorList = GridUtils.getCheckedValues(factorGrid, 0, FACTOR_DATA, true);
		EvaluationFactor evaluationFactor = checkedFactorList.get(0);
		addEvaluationFactor(evaluationFactor);
		evaluationStep.setScore(evaluationFactor.getFactorScore());
		evaluationStep.setWeight(evaluationFactor.getFactorScore().multiply(FactorUtils.getRealWeight(evaluationFactor.getFactorFinalWeight())));
		evaluationStep.setEvaluationParam(EvaluationStepEnum.PRICE_REVIEW_TYPE, PriceReviewTypeEnum.DETAIL);
	}
	
	@Override
	protected Point getInitialSize() {
		return new Point(600, 350);
	}
	
}
