package com.gx.obe.step.price.enums;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import com.gx.obe.util.utils.EnumUtils;
import com.gx.obe.util.utils.PredicateUtils;

/**
 * @Description: 报价明细表列枚举
 * @author chenxw
 */
public enum PriceDetailTableColEnum {
	
	/**
	 * @Description: 物料编码
	 */
	MATERIAL_CODE("materialCode", "物料编码", TypeEnum.OTHER),
	/**
	 * @Description: 项目单位
	 */
	NAME("name", "项目单位", TypeEnum.OTHER),
	/**
	 * @Description: 数量
	 */
	NUMBER("number", "数量", TypeEnum.OTHER),
	/**
	 * @Description: 未含税单价
	 */
	NO_TAX_SINGLE_PRICE("noTaxSinglePrice", "未含税单价", TypeEnum.PRICE),
	/**
	 * @Description: 含税单价
	 */
	SINGLE_PRICE("singlePrice", "含税单价", TypeEnum.PRICE),
	/**
	 * @Description: 含税合价
	 */
	PRICE("price", "含税合价", TypeEnum.PRICE),
	/**
	 * @Description: 得分权重
	 */
	SCORE_WEIGHT("scoreWeight", "得分权重", TypeEnum.OTHER);
	
	public enum TypeEnum {
		PRICE, OTHER;
	}
	
	/**
	 * @Description: keyMap
	 */
	public static Map<String, PriceDetailTableColEnum> MAP = EnumUtils.toMap(values(), PriceDetailTableColEnum::getKey);
	
	/**
	 * @Description: keyMap
	 */
	public static Map<String, PriceDetailTableColEnum> PRICE_MAP = EnumUtils.toMap(values(), PriceDetailTableColEnum::getKey, PredicateUtils.map(PriceDetailTableColEnum::getTypeEnum, Predicate.isEqual(TypeEnum.PRICE)));
	
	/**
	 * @Description: key列表缓存
	 */
	public static Map<TypeEnum, List<String>> keyCacheMap = new HashMap<>();
	
	private String key;
	private String value;
	private TypeEnum typeEnum;
	
	/**
	 * @author chenxw
	 * @param key
	 * @param value
	 * @param typeEnum
	 */
	PriceDetailTableColEnum(String key, String value, TypeEnum typeEnum) {
		this.key = key;
		this.value = value;
		this.typeEnum = typeEnum;
	}
	
	/**
	 * @Description: 是否包含指定类型指定key的枚举
	 * @author chenxw
	 * @param key
	 * @param typeEnum
	 * @return
	 */
	public static boolean contain(String key, TypeEnum typeEnum) {
		return keyCacheMap.computeIfAbsent(typeEnum, PriceDetailTableColEnum::getKeyListByType).stream().anyMatch(Predicate.isEqual(key));
	}
	
	/**
	 * @Description: 根据类型获取key列表
	 * @author chenxw
	 * @param typeEnum
	 * @return
	 */
	public static List<String> getKeyListByType(TypeEnum typeEnum) {
		return Arrays.stream(values()).filter(PredicateUtils.map(PriceDetailTableColEnum::getTypeEnum, Predicate.isEqual(typeEnum))).map(PriceDetailTableColEnum::getKey).collect(Collectors.toList());
	}
	
	public String getKey() {
		return key;
	}
	
	public String getValue() {
		return value;
	}
	
	public TypeEnum getTypeEnum() {
		return typeEnum;
	}
	
}
