package com.gx.obe.step.price.table;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.eclipse.swt.widgets.Composite;

import com.gx.obe.datastruct.tree.table.ParseTrees;
import com.gx.obe.datastruct.tree.table.TreeSpan;
import com.gx.obe.step.price.beans.PriceDetail;
import com.gx.obe.step.price.beans.PriceDetailEvalResult;
import com.gx.obe.web.entity.EvaluationFactorResult;
import com.gx.obe.web.entity.Supplier;

import de.kupzog.ktable.KTable;
import de.kupzog.ktable.model.CommonKtableCModel;

/**
 * @Description: 价格明细评审表格
 * @author chenxw
 */
public class PriceDetailTreeTable extends KTable {
	
	private CommonKtableCModel model;
	
	/**
	 * @Description: 供应商列表
	 */
	private final List<Supplier> supplierList;
	
	/**
	 * @Description: 解析树
	 */
	private final ParseTrees parseTrees;
	/**
	 * @Description: 普通表格列
	 */
	private final List<TableCol> tableColList = new ArrayList<>();
	
	/**
	 * @Description: 价格指标评审结果
	 */
//	private final Map<String, EvaluationFactorResult> factorResultMap;
	/**
	 * @Description: 得分排序编号map
	 */
//	private final Map<String, Integer> scoreSortNumberMap;
	
	public PriceDetailTreeTable(Composite parent, int style, List<Supplier> supplierList, List<PriceDetail> priceDetailList, List<PriceDetailEvalResult> evalResultList, List<EvaluationFactorResult> factorResultList) {
		super(parent, style);
		this.supplierList = supplierList;
		this.parseTrees = createParseTrees(priceDetailList, evalResultList);
		
//		this.factorResultMap = factorResultList.stream().collect(Collectors.toMap(EvaluationFactorResult::getSupplierId, Function.identity()));
//		ListSortUtils<EvaluationFactorResult> listSortUtils = new ListSortUtils<>(factorResultList, Comparator.comparing(EvaluationFactorResult::getEvaluationScore));
//		this.scoreSortNumberMap = factorResultList.stream().collect(Collectors.toMap(EvaluationFactorResult::getSupplierId, listSortUtils::getSortNo));
		
		initCommonKtableCModel();
	}
	
	/**
	 * @Description: 生成报价明细解析树
	 * @author chenxw
	 * @param priceDetailList
	 * @param evalResultList
	 */
	private ParseTrees createParseTrees(List<PriceDetail> priceDetailList, List<PriceDetailEvalResult> evalResultList) {
		priceDetailList = Optional.ofNullable(priceDetailList).orElseGet(ArrayList::new);
		evalResultList = Optional.ofNullable(evalResultList).orElseGet(ArrayList::new);
		Map<String, PriceDetailEvalResult> evalResultMap = evalResultList.stream().collect(Collectors.toMap(PriceDetailEvalResult::getDetailCode, Function.identity()));
		
		return new ParseTrees(priceDetailList.stream().map(t -> new PriceDetailTree(t, evalResultMap.get(t.getDetailCode()))).collect(Collectors.toList()));
	}
	
	/**
	 * @Description: 初始化表格模型
	 * @author chenxw
	 */
	private void initCommonKtableCModel() {
//		Function<Object, EvaluationFactorResult> evaluationFactorResultFun = factorResultMap::get;
//		Function<Object, Integer> scoreSortNumberFun = scoreSortNumberMap::get;
		
//		tableColList.add(new TableCol("总分", evaluationFactorResultFun.andThen(EvaluationFactorResult::getEvaluationScore).andThen(String::valueOf)));
//		tableColList.add(new TableCol("排序", scoreSortNumberFun.andThen(String::valueOf)));
		
		model = new CommonKtableCModel(this, 1, parseTrees.getHeight(), parseTrees.getLeafCount() + tableColList.size(), supplierList.size());
		
		for (TreeSpan treeSpan : parseTrees.getWideSpanList()) {
			model.addRowSpan(model.getFixedRowCount() + treeSpan.getLeafIndex(), treeSpan.getLevel(), treeSpan.getSpanCount());
		}
		
		model.addColSpan(0, 0, parseTrees.getHeight());
		for (int i = 0; i < tableColList.size(); i++) {
			model.addColSpan(model.getFixedRowCount() + parseTrees.getLeafCount() + i, 0, parseTrees.getHeight());
		}
		
		model.setRowHeightRender(row -> 30);
		model.setCellContentRender(this::receiveContent);
		this.setModel(model);
		this.getParent().layout();
	}
	
	/**
	 * @Description: 加载表格内容
	 * @author chenxw
	 * @param row
	 * @param col
	 * @return
	 */
	private String receiveContent(int row, int col) {
		if (col < model.getFixedColumnCount()) {
			if (row < model.getFixedRowCount()) {
				return "报价明细";
			}
			row -= model.getFixedRowCount();
			if (row < parseTrees.getLeafCount()) {
				return parseTrees.getITree(col, row).getName();
			}
			return tableColList.get(row - parseTrees.getLeafCount()).getName();
		}
		Supplier supplier = supplierList.get(col - model.getFixedColumnCount());
		if (row < model.getFixedRowCount()) {
			return supplier.getSupplierName();
		}
		row -= model.getFixedRowCount();
		if (row < parseTrees.getLeafCount()) {
			return parseTrees.get(row).getValue(supplier.getId());
		}
//		return tableColList.get(row - parseTrees.getLeafCount()).getValueFun().apply(supplier.getId());
		return "";
	}
	
}
