package com.gx.obe.step.price.utils;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Objects;

import com.gx.obe.util.utils.CollectionUtils;

/**
 * @Description: BigDecimal工具类
 * @author chenxw
 */
public class BigDecimalUtils {
	
	/**
	 * @Description: 平均值
	 * @author chenxw
	 * @param bigDecimals
	 * @return
	 */
	public static BigDecimal average(Collection<BigDecimal> bigDecimals) {
		CollectionUtils.notNull(bigDecimals);
		CollectionUtils.notContainNull(bigDecimals);
		return bigDecimals.stream().reduce(BigDecimal.ZERO, BigDecimal::add).divide(BigDecimal.valueOf(bigDecimals.size()), 10, BigDecimal.ROUND_HALF_UP);
	}
	
	/**
	 * @Description: 区间范围
	 * @author chenxw
	 * @param source
	 * @param min
	 * @param max
	 * @return
	 */
	public static BigDecimal withinScope(BigDecimal source, BigDecimal min, BigDecimal max) {
		Objects.requireNonNull(source);
		if (Objects.nonNull(min)) source = source.max(min);
		if (Objects.nonNull(max)) source = source.min(max);
		return source;
	}
	
	/**
	 * @Description: 替换
	 * @author chenxw
	 * @param source
	 * @param eqValue
	 * @param defValue
	 * @return
	 */
	public static BigDecimal replace(BigDecimal source, BigDecimal eqValue, BigDecimal defValue) {
		if (source == eqValue) return defValue;
		if (Objects.isNull(eqValue)) return source;
		if (Objects.isNull(source)) return null;
		if (source.compareTo(eqValue) == 0) return defValue;
		return source;
	}
	
}
