package com.gx.obe.step.price.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.gx.obe.business.entity.EvaluationStepFactor;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.components.core.enumeration.PriceReviewTypeEnum;
import com.gx.obe.datastruct.tree.Forest;
import com.gx.obe.step.price.enums.PriceDetailTableColEnum;
import com.gx.obe.step.service.StepFactorService;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.PredicateUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.evaluation.EvaluationFactorService;
import com.gx.obe.web.service.evaluation.EvaluationStepFactorService;

public interface IPriceEvalStep {
	
	EvaluationStepFactorService evaluationStepFactorService = new EvaluationStepFactorService();
	EvaluationFactorService evaluationFactorService = new EvaluationFactorService();
	
	boolean isAdd();
	
	/**
	 * @Description: 根据步骤绑定的指标过滤指标树
	 * @author chenxw
	 * @param factorForest
	 */
	default void filterForestByStepFactor(TenderProjectEntity tenderProject, EvaluationStep evaluationStep, Forest<EvaluationFactor> factorForest) {
		List<EvaluationStepFactor> evaluationStepFactorList = StepFactorService.getListByAuditType(tenderProject.getId(), EvaluationFactorEnum.AUDIT_PRICE);
		if (CollectionUtils.isNull(evaluationStepFactorList)) {
			return;
		}
		Stream<String> factorIdStream = evaluationStepFactorList.stream().map(EvaluationStepFactor::getFactorId);
		if (!this.isAdd()) {
			List<Object> factorIdLsitByStepId = evaluationStepFactorService.getFactorIdLsitByStepId(evaluationStep.getId());
			factorIdStream = factorIdStream.filter(t -> !factorIdLsitByStepId.contains(t));
		}
		List<String> factorIdList = factorIdStream.collect(Collectors.toList());
		factorForest.filterPath(t -> !factorIdList.contains(t.getId()));
	}
	
	static boolean isCreatePriceStep(TenderProjectEntity tenderProject, List<EvaluationStepFactor> evaluationStepFactorList, String priceReviewType) {
		List<EvaluationFactor> priceEvalFactorList = evaluationFactorService.getAllEvaluationFactorList(tenderProject.getId(), EvaluationFactorEnum.AUDIT_PRICE);
		if (CollectionUtils.isNull(priceEvalFactorList)) {
			return false;
		}
		List<EvaluationFactor> list = new ArrayList<>(priceEvalFactorList);
		Forest<EvaluationFactor> factorForest = new Forest<>(list);
		if (CollectionUtils.isNotNull(evaluationStepFactorList)) {
			List<String> factorIdList = evaluationStepFactorList.stream().map(EvaluationStepFactor::getFactorId).collect(Collectors.toList());
			factorForest.filterPath(t -> !factorIdList.contains(t.getId()));
		}
		IPriceEvalStep.filterForestByReviewType(factorForest, priceReviewType);
		return !list.isEmpty();
	}
	
	/**
	 * @Description: 根据价格步骤的评审类型过滤指标树
	 * @author chenxw
	 * @param factorForest
	 */
	static void filterForestByReviewType(Forest<EvaluationFactor> factorForest, String priceReviewType) {
		factorForest.filterPath(PredicateUtils.map(EvaluationFactor::getBidPriceCode, getBidPriceCodePredicate(priceReviewType)));
	}

	static Predicate<String> getBidPriceCodePredicate(String priceReviewType) {
		switch (priceReviewType) {
			case PriceReviewTypeEnum.PRICE:
				return t -> !PriceDetailTableColEnum.contain(t, PriceDetailTableColEnum.TypeEnum.PRICE);
			case PriceReviewTypeEnum.DETAIL:
				return t -> PriceDetailTableColEnum.contain(t, PriceDetailTableColEnum.TypeEnum.PRICE);
		}
		return t -> false;
	}
	
}
